{# Base layout for all pages

## Analytics

The service https://plausible.io is used to gather simple
and privacy-friendly analytics for the site. The dashboard can be accessed
at https://analytics.scientific-python.org/docs.scipy.org
The Scientific-Python community is managing the account.

#}

{% extends "!layout.html" %}

{# ================ #}
{# Extending Header #}
{# ================ #}

{% block extrahead %}
    <script defer data-domain="docs.scipy.org" src="https://views.scientific-python.org/js/script.js"></script>

    <!-- My custom JS -->
    <script type="text/javascript" src="../_static/js/custom-pydata.js"></script>

    <!-- Syntax highlighting for BibTex code blocks -->
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/prism/1.23.0/themes/prism-solarizedlight.min.css"/>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/prism/1.23.0/prism.min.js"></script>
    <script src="https://cdn.jsdelivr.net/npm/prismjs-bibtex@2.0.1/prism-bibtex.min.js"></script>

    {{ super() }}
{% endblock %}

{# ================ #}
{# Extending Footer #}
{# ================ #}

{% block footer %}

    <!-- Adobe Embed API -->
    {# <script src="https://documentcloud.adobe.com/view-sdk/viewer.js"></script> #}
    <script src="https://documentcloud.adobe.com/view-sdk/main.js"></script>

{% endblock %}
