/* override table width restrictions */
.wy-table-responsive table td, .wy-table-responsive table th {
    /* !important prevents the common CSS stylesheets from
       overriding this as on RTD they are loaded after this stylesheet */
    white-space: normal !important;
}

.wy-table-responsive {
    overflow: visible !important;
}

/* Code block colors */
.highlight {
    background: #eeffcc;
}

/* Highlighted lines in code blocks */
.highlight .hll {
    background-color: #e8e8e8;
}

/* Push the math equation label to the right */
.math {
    text-align: left;
}
.eqno {
    float: right;
}

/* .wy-nav-content { */
    /* height: 100%; */
    /* max-width: 1500px; */
    /* max-width: 60% !important; */
    /* margin: auto; */
    /* background-color: #ffffff; */
/* } */

/* .wy-nav-content-wrap { */
/*     /* background color of wrap around main content*/ */
/*     background-color: white; */
/* } */
