# SPDX-FileCopyrightText: Copyright 2021, Siavash Ameli <sameli@berkeley.edu>
# SPDX-License-Identifier: BSD-3-Clause
# SPDX-FileType: SOURCE
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the license found in the LICENSE.txt file in the root directory
# of this source tree.


# =======
# Imports
# =======

import sympy


# ==========================
# Declare symbolic variables
# ==========================

n = sympy.symbols('n', integer=True, positive=True)
t = sympy.symbols('t', real=True, positive=True)
